/*MAND3D.C for Turbo C/C++ Version 3 by David Walter, LONDON UK,
phone/fax +44 0181 360 2689.   Compuserve 100013,2775.

I want to incorporate STEREO PAIRS into FRACTINT - IDEALLY SUCH THAT 
AFTER VIEWING A PICTURE ON THE SCREEN, THE PROGRAM WOULD ROTATE IT
BY 90 DEGREES AND PRODUCE THE STEREO PAIRS.  THE SIZE OF EACH PICTURE 
CAN THEN BE INCREASED TO FILL HALF THE SCREEN, AND, BY CROSSING THE EYES,
CAN BE SEEN AS A 3D PICTURE.

Mset stereo pairs using equations from the book 'Strange Atractors'
by JC Sprott pub M&T Books, p149; 
program based on MANDEL.C P490 Turbo C by Lafore.

This program is designed specifically for a 14" 640x480 pixel screen.
It generates a stereo pair of M set defined by "left", "top" etc.

Each picture is surrounded by a rectangle which gives you an idea of
how the left and right hand image pixels have have been shifted by "xshift".
Each rectangle has a little circle over it as an aid to focussing the 
eyes.  

I think the simplest way of seeing the 3D image is as follows:
(you may try jumping to STEP 4 straight away)

1) Get a pen point and hold it in front of the screen.

2) Shut your right eye and look at the right circle with your left eye; 
	 move the pen point until it is in line with your left eye and 
	 the right circle.  

3) Shut your left eye and look at the left circle with your right eye; 
	 move the pen point until it is in line with your right eye and 
	 the left circle.
	 
4) With both eyes open stare at the point of the pen and move the
	 pen until the left eye and the point and the right circle are in line,
				 and the right eye and the point and the left  circle are in line.
	 You should then see THREE circles; the one in the middle is the one you
	 should try to shift your concentration to,  whilst keeping your eyes 
	 firmly crossed,  and not laughing with those looking at you in amazement.
	 NOW GRADUALLY look at the central picture, which should be in glorious 3D.   
	 	
	 The other way is to look at the left picture with the left eye,
	 and the right picture with the right eye.  But this way the maximum
	 picture width is the same as the distance apart of your eyes - 
	 a rather small 6.5 cms.

	 Enjoy it.
*/


#include <graphics.h>
#include <conio.h>
#define MAXCOUNT 1023    /*max # of iterations */

main()
{
	int gdriver = DETECT, gmode;
	int x, y, screenwidth, screenheight, count;
	double xscale, yscale, left, top, xside, yside, zx, zy, cx, cy, tempx;
	double EYESEPARATION, VIEWDIST, DISTAPART;
	double PICWIDTH, PICHEIGHT, MAXDEPTH, depth, xshift;
	initgraph(&gdriver,&gmode,"..\\bgi");

	screenwidth = getmaxx();
	screenheight = getmaxy();

/* The "left" and "right"  locate the picture - in the real world xy plane 
	 not the best way but you can change it easily. */
	left = -0.844211314;   top = 0.227532065; 
	xside = 0.009355936; yside = -0.007017524;

	EYESEPARATION = 250; /*180 PIXELS = 6.5 CMS ON 640X480 SCREEN -
	THE EYE SEPARATION IS ALSO EFFECTIVELY THE WIDTH OF EACH PICTURE */
	DISTAPART = 6;   /*27.8 PIXELS = 1 CM */
	VIEWDIST = 1800;  /* ideal viewing distance ten times eye separation 
	= 65 cm in pixels = 1807 PIXELS */	
	PICWIDTH = EYESEPARATION - DISTAPART; /*WIDTH  OF EACH PICTURE*/
	PICHEIGHT = PICWIDTH/1.3;             /*HEIGHT OF EACH PICTURE*/
	MAXDEPTH = 4*PICWIDTH;   /*relate max feature depth to width of pic*/
	xscale = xside/PICWIDTH; yscale = yside/PICHEIGHT;

	/* put rectangles round pics & 1 pixel bigger to act as reference */
	rectangle(screenwidth/2 - EYESEPARATION/2 -PICWIDTH/2 - 1, 
						screenheight/2 -PICHEIGHT/2 - 1,
						screenwidth/2 - DISTAPART/2 + 1, 
						screenheight/2 + PICHEIGHT/2 +1);

	rectangle(screenwidth/2 + DISTAPART/2 - 1, 
						screenheight/2 - PICHEIGHT/2 - 1,
						screenwidth/2 + DISTAPART/2 + PICWIDTH + 1, 
						screenheight/2 + PICHEIGHT/2 +1);

	/* draw circles above pics to direct eyes at */
	circle(screenwidth/2 - EYESEPARATION/2, 
	screenheight/2 - PICHEIGHT/2 - 10, 5);

	circle(screenwidth/2 + EYESEPARATION/2, 
	screenheight/2 - PICHEIGHT/2 - 10, 5);

 /* Calculations for pictures from Mandelbrot Set  */
	for(y=0; y<=PICHEIGHT; y++)
		{
		for(x=0; x<=PICWIDTH; x++)
			{
			cx=x*xscale +left;
			cy=y*yscale +top;
			zx=cx;  zy=cy;
			count=1;

			while(zx*zx+zy*zy<4 && count<MAXCOUNT  && !kbhit() )/* any key exits*/
				{
				tempx =zx*zx - zy*zy +cx;
				zy=2*zx*zy+cy;
				zx=tempx;
				count++;
				}
			depth = count*MAXDEPTH/MAXCOUNT;/* 3D depth proportional to count */

			/* shift pixel proportional to depth; if shift = 0 pictures fit
			exactly inside rectangles; if not, you can see amount shifted*/
			xshift = EYESEPARATION*depth/(2*VIEWDIST);

			/*left hand picture: xshift +ve for receding depth
			with eyes crossed; if xshift -ve valleys become hills.*/
			putpixel(x + xshift + screenwidth/2 - EYESEPARATION/2 - PICWIDTH/2, 
							y +  screenheight/2 - PICHEIGHT/2, count);

      /*right hand picture: */
			putpixel(x - xshift + screenwidth/2 + DISTAPART/2, 
							y + screenheight/2 -PICHEIGHT/2, count);

			if(kbhit() )
				exit(0);
				 }
		}
			getche();
			closegraph();
  return(0);
}